 #include <stdint.h>
#include <stdbool.h>
#include "inc/hw_memmap.h"
#include "inc/hw_types.h"
#include "driverlib/gpio.h"
#include "driverlib/pin_map.h"
#include "driverlib/sysctl.h"
#include "driverlib/uart.h"
#include "driverlib/adc.h"
#include "driverlib/timer.h"
#include "driverlib/interrupt.h"


uint16_t i,nsample=0,nmed,nmedido,tcarga,tcar,Irradiancia,Temperatura;  //tcarga por defecto en milisegundos, nmed = numero de muestras
uint32_t ui32ADC0Value[1],ui32ADC1Value[1],dataADC0[2500],dataADC1[2500],G[20],T[20];


void setup()
{
Serial.begin(115200); //Config puerto serie para 115200
Serial.setTimeout(8); //Tiempo máximo de lectura del puerto serie en ms (para leer el valor de tiempo de carga)

SysCtlPeripheralEnable(SYSCTL_PERIPH_ADC0); //Activo ADC0
SysCtlPeripheralEnable(SYSCTL_PERIPH_ADC1); //Activo ADC1
SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);  //Activo PUERTOS "A"
SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);  //Activo PUERTOS "E"
SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);  //Activo PUERTOS "D"
SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOF);  //Activo PUERTOS "F"
ADCSequenceConfigure(ADC0_BASE, 3, ADC_TRIGGER_TIMER, 0); //Config ADC0 para disparo por temporizador secuenciador 3
ADCSequenceConfigure(ADC1_BASE, 3, ADC_TRIGGER_TIMER, 0); //Config  ADC1 para disparo por temporizador secuenciador 3
ADCSequenceConfigure(ADC0_BASE, 2, ADC_TRIGGER_PROCESSOR, 0); //Config ADC0 disparo por processor secuenciador 2
ADCSequenceConfigure(ADC1_BASE, 2, ADC_TRIGGER_PROCESSOR, 0); //Config  ADC1 disparo por processor secuenciador 2
GPIOPinTypeADC(GPIO_PORTE_BASE,GPIO_PIN_2); //Config  Puerto "E2"(CH1) para medida ADC (I)
GPIOPinTypeADC(GPIO_PORTE_BASE,GPIO_PIN_1); //Config  Puerto "E1"(CH2) para medida ADC (T)
GPIOPinTypeADC(GPIO_PORTD_BASE,GPIO_PIN_3); //Config  Puerto "D3"(CH4) para medida ADC (G)
GPIOPinTypeADC(GPIO_PORTD_BASE,GPIO_PIN_2); //Config  Puerto "D2"(CH5) para medida ADC (V)
ADCSequenceStepConfigure(ADC0_BASE, 3, 0, ADC_CTL_CH5 | ADC_CTL_IE |ADC_CTL_END); //Config  ADC0 para medir por puerto CH5 = D2 secuenciador 3(Tensión)
ADCSequenceStepConfigure(ADC1_BASE, 3, 0, ADC_CTL_CH1 | ADC_CTL_IE |ADC_CTL_END); //Config  ADC1 para medir por puerto CH1 = E2 secuenciador 3(Corriente)
ADCSequenceStepConfigure(ADC0_BASE, 2, 0, ADC_CTL_CH4 | ADC_CTL_IE |ADC_CTL_END); //Config  ADC0 para medir por puerto CH4 = D3 secuenciador 2(Irradiancia)
ADCSequenceStepConfigure(ADC1_BASE, 2, 0, ADC_CTL_CH2 | ADC_CTL_IE |ADC_CTL_END); //Config  ADC1 para medir por puerto CH2 = E1 secuenciador 2(Temperatura)
ADCSequenceEnable(ADC0_BASE, 3);  //Activo secuencia 3 para ADC0
ADCSequenceEnable(ADC1_BASE, 3);  //Activo secuencia 3 para ADC1
ADCSequenceEnable(ADC0_BASE, 2);  //Activo secuencia 2 para ADC0
ADCSequenceEnable(ADC1_BASE, 2);  //Activo secuencia 2 para ADC1
ADCHardwareOversampleConfigure(ADC0_BASE, 32);  //Config  ADC0 para sobremuestreo (mayor precisión) 
ADCHardwareOversampleConfigure(ADC1_BASE, 32);  //Config  ADC1 para sobremuestreo (mayor precisión) 
SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER0); //Activo timer0 para trigger de los ADC
TimerConfigure(TIMER0_BASE, TIMER_CFG_PERIODIC);  //Configuro timer0 como Full-width periodic timer
TimerControlTrigger(TIMER0_BASE, TIMER_A, true);  //Configuro timer0 como Timer_A y habilito la interrupción del ADC por timer
ADCIntRegister(ADC0_BASE, 3, ADCIntHandler);  //Configuro la secuencia 3 del ADC0 como la función de interrupción ADCIntHandler
ADCIntClear(ADC0_BASE, 3);  //Limpio el estado de interrupcion
ADCIntEnable(ADC0_BASE, 3); //Habilito la interrupción en ADC0 secuencia 3
GPIOPinTypeGPIOOutput(GPIO_PORTF_BASE, (GPIO_PIN_1)); //Configuracion como salida de PF1 (descarga)
GPIOPinTypeGPIOOutput(GPIO_PORTF_BASE, (GPIO_PIN_4)); //Configuracion como salida de PF4 (precarga)
GPIOPinTypeGPIOOutput(GPIO_PORTE_BASE, (GPIO_PIN_3)); //Configuracion como salida de PE3 (capacitor 2)
GPIOPinTypeGPIOOutput(GPIO_PORTD_BASE, (GPIO_PIN_0)); //Configuracion como salida de PD0 (módulo)
GPIOPinTypeGPIOOutput(GPIO_PORTA_BASE, (GPIO_PIN_4)); //Configuracion como salida de PA4 (sincronismo)
GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_1, 0); //PF1 en estado bajo
GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_4, 0); //PF4 en estado bajo
GPIOPinWrite(GPIO_PORTD_BASE, GPIO_PIN_0, 0); // PD0 en estado bajo
GPIOPinWrite(GPIO_PORTE_BASE, GPIO_PIN_3, 0); // PE3 en estado bajo
GPIOPinWrite(GPIO_PORTA_BASE, GPIO_PIN_4, 0); // PA4 en estado bajo
}

void loop()
{
  if(Serial.available())
  {
    switch(Serial.read())
    {
      case '1': //Medida de irradiancia y temperatura
        G[0]=0;
        T[0]=0;
        ADCProcessorTrigger(ADC0_BASE, 2);  //Realiza la medida en ADC0 secuenciador 2 (Irradiancia)
        ADCProcessorTrigger(ADC1_BASE, 2);  //Realiza la medida en ADC1 secuenciador 2 (Temperatura)
        delay(1);
        ADCSequenceDataGet(ADC0_BASE, 2, ui32ADC0Value);  //Obtiene la medida en ADC0 secuenciador 2 (Irradiancia)  
        ADCSequenceDataGet(ADC1_BASE, 2, ui32ADC1Value);  //Obtiene la medida en ADC1 secuenciador 2 (Temperatura)
        G[0] = ui32ADC0Value[0];
        T[0] = ui32ADC1Value[0]; 
      break;  
     case '2': //Irradiancia por puerto serie
      Serial.println(G[0]);
    break; 
     case '3': //Temperatura por puerto serie
      Serial.println(T[0]);  
    break; 
      case '4': //Medida de tensión y corriente
        while(!Serial.available());
        if(Serial.available()>0)
       {
        int nmedido=Serial.parseInt();  //obtención de número de muestras por puerto serie 
        nmed=nmedido;
        delay(1);
       }
        while(!Serial.available());
        if(Serial.available()>0)
       {
        int tcar=Serial.parseInt(); //obtencion del tiempo de carga por puerto serie 
        tcarga=tcar;
        delay(1);
       }
        GPIOPinWrite(GPIO_PORTA_BASE, GPIO_PIN_4, 16);  //PIN PA4 en estado alto para el posterior sincronismo, se dispara con flanco descendente 
        TimerLoadSet(TIMER0_BASE, TIMER_A, 80000/nmed*tcarga);  //configurar temporizador, tcarga en ms
        ADCIntEnable(ADC0_BASE, 3); //Interrupción activa en el ADC  
       GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_1, 2); //Activo descarga PIN PF1 por seguridad
       delay(100);  //retardo de 1 seg para descarga
       GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_1, 0); //desactivo descarga PIN PF1
       delay(100);
        GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_4, 16);  //activo precarga PF4
        delay(100);
      GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_4, 0); //desactivo precarga PF4
        delay(100);
        GPIOPinWrite(GPIO_PORTD_BASE, GPIO_PIN_0, 1); //Activo módulo PIN PD0
        TimerEnable(TIMER0_BASE, TIMER_A);  //Inicio de temporización
        GPIOPinWrite(GPIO_PORTA_BASE, GPIO_PIN_4, 0);       //Activo sincronismo PIN PA4, paso de estado alto a bajo    
      break;
      case '5': //Datos tensión  por puerto serie
        for(i=0;i<nmed;i++)
        {    
          Serial.println(dataADC0[i]);
          delay(1);
        }      
      break;
      case '6': //datos corriente por puerto serie
        for(i=0;i<nmed;i++)
        {            
          Serial.println(dataADC1[i]);
          delay(1);        
        }       
      break;
    } 
  }
}
///////////////////////////////////////////////////////////////////////////////////Interrupción ADC/////////////////////////////////////////////////////////////////////////
void ADCIntHandler(void) {
     ADCSequenceDataGet(ADC0_BASE, 3, ui32ADC0Value);  //Obtiene la medida en ADC0 secuenciador 3(Tension)
     ADCSequenceDataGet(ADC1_BASE, 3, ui32ADC1Value);  //Obtiene la medida en ADC1 secuenciador 3(Corriente) 
     dataADC0[nsample] = ui32ADC0Value[0]; //Guarda el valor de tension
     dataADC1[nsample] = ui32ADC1Value[0]; //Guarda el valor de corriente
     nsample++;
     ADCIntClear(ADC0_BASE, 3);  //Limpia la interrupcion
    if(nsample == nmed) //Si ha realizado todas las medidas... 
    {   
      GPIOPinWrite(GPIO_PORTD_BASE, GPIO_PIN_0, 0); //desactivo módulo PIN PD0      
      TimerDisable(TIMER0_BASE, TIMER_A); //Fin de temporización
      nsample=0;  //Reinicia el contador de medidas
      //ADCIntClear(ADC0_BASE, 3);
      ADCIntDisable(ADC0_BASE, 3);  //Desactiva la interrupcion     
      delay(100);  
      GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_1, 2); //Activo descarga PIN PF1 
      delay(1000);  //retardo de 1 seg para descarga
      GPIOPinWrite(GPIO_PORTF_BASE, GPIO_PIN_1, 0); //desactivo descarga PIN PF1 por seguridad  
      Serial.println(1);  //enviar el número 1 por puerto serie indicando fin de la medición
  } 
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

